#!/usr/bin/env python
# encoding: utf-8
"""
$Id$

Purpose: Run post prosses on products produced by ADL

public methods:

Copyright (c) 2011 University of Wisconsin Regents.
Licensed under GNU GPLv3.
"""
import os
import sys
import logging
import glob
import traceback
import xml.etree.ElementTree as ET
import shutil
import time
import sqlite3
import fnmatch
from common.adl_common import sh
from common.adl_common import DDS_PRODUCT_FILE, CSPP_RT_HOME
from common.adl_geo_ref import write_geo_ref
from common.adl_geo_ref import GEO_GUIDE
from common.adl_dms_setup import AdlDmsSetup 
from subprocess import CalledProcessError
from multiprocessing import Pool
from common.shortnames import Shortnames

LOG = logging.getLogger(__name__)

class ProductOperations:
    _adl_dms_setup = AdlDmsSetup()
    _work_dir = None
    _instrument = None
    _spacecraft = None
    _shortname = None
    _product_output_dir = None


    def __init__(self, instrument, spacecraft):
        
        self._instrument = instrument
        self._spacecraft = spacecraft
        self._shortname = Shortnames()

    def add_granule_data(self, work_dir, product_output_dir=None):
        self._work_dir = work_dir

        if product_output_dir == None:
            self._product_output_dir = os.path.dirname(self._work_dir)
        else:
            self._product_output_dir = product_output_dir


    def check_products(self, gran_id, product_shortnames, chunk_size=1):
        """ Checks that proper products were produced.
        If product h5 file was produced blob and lutasc files are deleted
        If product was not produced files are left alone
        """
        # look for all products that should have been produced
        # convert the products to H5
        LOG.info("Check for Granule: " + gran_id + " products.")
        start_time = time.time()
        LOG.debug("<< check_for_products  >>")
        # all = glob.glob(os.path.join(work_dir, '*'))
        problem = False
        total = 0
        good = 0

        for short_name in sorted(product_shortnames):
            prod_id = self._shortname.SHORTNAME_2_PRODUCTID[short_name]
            #LOG.debug('look %s %s' % (os.path.join(self._work_dir, '%s*.h5' % prod_id),short_name))
            gran_list = glob.glob(os.path.join(self._work_dir, '%s*' % prod_id))
            if len(gran_list) < 1:
                total = total + 1
                try:
                    LOG.warning(
                        "Problem : " + gran_id + " " + short_name + " No product produced. " + prod_id)
                    problem = True
                except KeyError:
                    LOG.warning("Short name not in product map:" + short_name)
            else:
                for sdr_name in gran_list:
                    try:
                        total = total + 1
                        LOG.debug("Product: " + sdr_name + " produced.")
                        if sdr_name.find("_d1958") != -1:
                            LOG.warning(
                                "DELETE File with bad year: " + sdr_name)
                            if os.path.exists(sdr_name):
                                os.remove(sdr_name)
                            problem = True
                        else:
                            good += 1
                    except KeyError:
                        LOG.error(
                            "Key error on blob property " + 'ObservedStartTime' + " or " + short_name)
        LOG.info(str(good) + " out of " + str(total) +
                 " products produced for granule " + gran_id)
        end_time = time.time()
        elapsed_time = end_time - start_time
        LOG.debug("<< Check Products Elapsed Time: %f >>" % elapsed_time)
        return problem

    def rename_products(self, product_ids):
        """ Checks that proper products were produced.
        If product h5 file was produced blob and lutasc files are deleted
        If product was not produced files are left alone
        """
        #dest = os.path.dirname(work_dir)
        good = True
        for product_id in product_ids:
            files = glob.glob(os.path.join(
                self._work_dir, '%s*.h5' % product_id))

            for sdrproduct in files:

                if sdrproduct.find("_d1958") != -1:
                    LOG.warning("IGNORE File with bad year: " + sdrproduct)
                else:
                    new_name = os.path.basename(
                        sdrproduct.replace('_cspu_', '_cspp_'))
                    new_name = os.path.basename(
                        new_name.replace('_pop', '_dev'))
                    new_name = os.path.basename(
                        new_name.replace('_ops', '_dev'))

                    shutil.move(sdrproduct, os.path.join(
                        self._work_dir, new_name))
                    LOG.debug('Product %s ' % sdrproduct)

            if len(files) < 1:
                LOG.debug('Product %s missing. %d products produced.' %
                          (product_id, len(files)))
                good = False

        return good

    def _find(self,pattern, path):
        result = []
        for root, dirs, files in os.walk(path):
            for name in files:
                if fnmatch.fnmatch(name, pattern):
                    result.append(os.path.join(root, name))
        return result

    from time import time
    def _remove_science_SDR(self,db_path,granule_id):
        """
        """
        start_time = time.time()
        try: 

            conn = self._adl_dms_setup.get_global_databasename_connection()
            end_time = time.time()
            delta = end_time - start_time
            LOG.debug("Connection lock %s"%str(delta)) 
            cursor = conn.cursor()

            #cmd = 'SELECT  URID FROM TDMS_COREUR_CORE_URS where DATASET_TYPE_TAG like "SDR" and GRANULE_ID LIKE "%s"'%granule_id
            cmd = 'SELECT  URID FROM TDMS_COREUR_CORE_URS where GRANULE_ID LIKE "%s"'%granule_id
            LOG.debug(cmd)
            cursor.execute(cmd)

            urids=cursor.fetchall()
            #LOG.debug(pp.pformat(urids))

            conn.commit()
            conn.close()

            path=os.path.join(db_path,"db","DmSm","SDR")
            for urid in urids :
                rem=self._find('%s*'%urid, path)
                for g in rem:
                    os.remove(g)
                #LOG.debug(pp.pformat(rem))
                
            path=os.path.join(db_path,"db","DmSm","GEO")
            for urid in urids :
                rem=self._find('%s*'%urid, path)
                for g in rem:
                    os.remove(g)
                #LOG.debug(pp.pformat(rem))    
        except :
            end_time = time.time()
            delta = end_time - start_time
            LOG.warning("Possible disk contention. You might try to reduce the -p value. %s"%str(delta))
 
    def move_products_to_work_directory(self, product_ids, optional=None):
        """ Checks that proper products were produced.
        If product h5 file was produced blob and lutasc files are deleted
        If product was not produced files are left alone
        """
        notifydest = os.path.dirname(self._work_dir)
        dest = self._product_output_dir

        chunks = 0
        option_chunks = 0

        # once=False
        product_start_time = None
        for product_id in product_ids:
            files = glob.glob(os.path.join(
                self._work_dir, '%s*.h5' % product_id))
            LOG.debug('chunks %d done for %s' % (len(files), product_id))
            if chunks < len(files):
                chunks = len(files)

            for sdrproduct in files:
                product_start_time = os.path.basename(sdrproduct).split("_")[3]
                #LOG.info("product start %s"%product_start_time)
                if sdrproduct.find("_d1958") != -1:
                    LOG.warning("IGNORE File with bad year: " + sdrproduct)
                else:
                    new_name = os.path.basename(
                        sdrproduct.replace('_cspu_', '_cspp_'))
                    new_name = os.path.basename(
                        new_name.replace('_pop', '_dev'))

                    new_name = os.path.basename(
                        new_name.replace('_ops', '_dev'))

                    shutil.move(sdrproduct, dest, new_name)

        if optional is not None:
            for opt in optional:
                files = glob.glob(os.path.join(self._work_dir, '%s*.h5' % opt))
                if option_chunks == 0:
                    option_chunks = len(files)
                for sdrproduct in files:

                    new_name = os.path.basename(
                        sdrproduct.replace('_cspu_', '_cspp_'))
                    new_name = os.path.basename(
                        new_name.replace('_pop', '_dev'))

                    new_name = os.path.basename(
                        new_name.replace('_ops', '_dev'))

                    shutil.move(sdrproduct, dest, new_name)
                    LOG.debug('Product %s ' % sdrproduct)

            if len(files) < 1:
                LOG.debug('Product %s missing. %d products produced.' %
                          (product_id, len(files)))

        granule_id = os.path.basename(self._work_dir)
        self._remove_science_SDR(dest, granule_id)
        LOG.debug('chunks %s option_chunks %s' % (chunks, option_chunks))

        try:
            if product_start_time != None:
                created_dir = os.path.join(
                    notifydest, "C_%s" % product_start_time)
                move_dir = os.path.join(notifydest, "CSPP_SDR_GRAN_%s_%s_%s" % (
                    self._instrument, self._spacecraft, product_start_time))
                os.makedirs(created_dir)
                os.rename(created_dir, move_dir)
        except:
            pass

        return chunks, option_chunks

    def add_geo_attribute_to_h5(self, msg, shortnames):
        """ Adds GEO_REF property to produced granules
        """
        added = 0

        LOG.info("Add GEO_REF property to : " + msg + " products.")

        for short_name in sorted(shortnames):

            prod_id = self._shortname.SHORTNAME_2_PRODUCTID[short_name]

            LOG.debug('look %s' % os.path.join(
                self._work_dir, '%s*.h5' % prod_id))
            gran_list = glob.glob(os.path.join(self._work_dir, '%s*' % prod_id))

            geo_prod = None
            for pat, stem, refid in GEO_GUIDE:
                if pat in prod_id:
                    geo_prod = stem

            # if len(gran_list) == 1:
            for sdr_name in gran_list:
                try:
                    sname = os.path.basename(sdr_name).split("_")

                    LOG.debug("GEO Product: " + sdr_name + " produced.")
                    LOG.debug(
                        "Oberserved : " + sdr_name + " short_name: " + short_name)
                    if geo_prod is not None:
                        geo_name = geo_prod + "*%s*.h5" % sname[3]
                        gnames = glob.glob(
                            os.path.join(self._work_dir, geo_name))

                        if len(gnames) > 0:
                            geo_filename = gnames[0]

                            write_geo_ref(
                                sdr_name, geo_filename=geo_filename)
                            added += 1
                except KeyError:
                    LOG.debug("Key error on blob property " + 'ObservedStartTime' +
                              " or " + short_name)
        if added > 0:
            LOG.debug("Added N_GEO_Ref properties to " + str(added) +
                      " files for " + msg + " products")
